@@ -10,7 +10,7 @@ from .encoders import *
 ##################################### BestSingleModel ################################################
 
 
-class BestSingleModel(XGBoost):
+class BestSingleModel(LightGBM):
     """
     Trying to find which model work best on our data
     Args:
@@ -55,6 +55,7 @@ class BestSingleModel(XGBoost):
 
     def opt(self, 
         timeout=1000, 
+        iterations=None,
         early_stoping=100, 
         cold_start=100,
         direction='maximize',
@@ -93,7 +94,8 @@ class BestSingleModel(XGBoost):
             timeout, 
             early_stoping, 
             feature_selection,
-            verbose)
+            iterations=iterations,
+            verbose=verbose,)
         return(history)
 
     def _predict_preproc_model(self, model_cfg, model,):
@@ -288,7 +290,7 @@ class AutoML(BestSingleModel):
         # calc ~ time for opt
         #min_stack_model_timeout = 1000
         predict_timeout = 30*stack_top # time for predict
-        select_models_timeout = (timeout - predict_timeout - 300)
+        select_models_timeout = (timeout - predict_timeout - 200)
         if select_models_timeout < 200:
             raise Exception(f"Please give me more time to optimize or reduce the number of stack models ('stack_top')")
 
@@ -342,20 +344,16 @@ class AutoML(BestSingleModel):
         predicts_1 = model.predict(models_cfgs=stack_models_1_cfgs)
         
         # Score:
+        score_mean_models_1 = self.metric(self._data.y_train, predicts_1['predict_train'].mean())
         if verbose > 0:
-            score_mean_models = self.metric(self._data.y_train, predicts_1['predict_train'].mean())
             print(f'\n Models_1 Mean {self.metric.__name__} Score Train: ', 
-                round(score_mean_models, self._metric_round))
+                round(score_mean_models_1, self._metric_round))
             time.sleep(0.1) # clean print 
 
         #############################################################
         # STEP 2
         # Model 2
         model_2 = LinearModel(databunch=self._data,
-                                opt_lvl=self._opt_lvl,
-                                cv=self._cv,
-                                score_cv_folds = self._score_cv_folds,
-                                auto_parameters = self._auto_parameters,
                                 metric=self.metric,
                                 direction=self.direction,
                                 metric_round=self._metric_round,
@@ -366,8 +364,12 @@ class AutoML(BestSingleModel):
 
         # Opt
         history_2 = model_2.opt(
-            timeout=300, 
-            auto_parameters=auto_parameters,
+            iterations=50, 
+            cv_folds=10,
+            score_cv_folds = 5,
+            opt_lvl=2,
+            auto_parameters=False,
+            cold_start=25,
             feature_selection=feature_selection,
             verbose= (lambda x: 0 if x <= 1 else 1)(verbose), )
 
@@ -381,17 +383,27 @@ class AutoML(BestSingleModel):
         predicts_2 = model_2.predict(models_cfgs=stack_model_2_cfgs)
         
         # Score:
+        score_mean_models_2 = self.metric(self._data.y_train, predicts_2['predict_train'].mean())
         if verbose > 0:
-            score_mean_models = self.metric(self._data.y_train, predicts_2['predict_train'].mean())
             print(f'\n Models_2 Mean {self.metric.__name__} Score Train: ', 
-                round(score_mean_models, self._metric_round))
+                round(score_mean_models_2, self._metric_round))
             time.sleep(0.1) # clean print 
 
         ###############################################################
         # STEP 3
-        self.stack_models_predicts = pd.concat([predicts_1, predicts_2], ignore_index=True, sort=False)
-        self.stack_models_cfgs = pd.concat([stack_models_1_cfgs, stack_model_2_cfgs], ignore_index=True, sort=False)
-
+        if self.direction is 'maximize':
+            add_model_2 = (score_mean_models_1 <= (score_mean_models_2+(score_mean_models_1/10)))
+        else:
+            add_model_2 = (score_mean_models_1 >= (score_mean_models_2-(score_mean_models_1/10)))
+        
+        if add_model_2:
+            self.stack_models_predicts = pd.concat([predicts_1, predicts_2], ignore_index=True, sort=False)
+            self.stack_models_cfgs = pd.concat([stack_models_1_cfgs, stack_model_2_cfgs], ignore_index=True, sort=False)
+        else:
+            self.stack_models_predicts = predicts_1
+            self.stack_models_cfgs = stack_models_1_cfgs
+        
+            
         pred_test = self.stack_models_predicts['predict_test'].mean()
         pred_train = self.stack_models_predicts['predict_train'].mean()
 
